; ------------------------------
;	DOOM DSP code
; ------------------------------

; ---------------------------------------------
;	Quelques definitions
; ---------------------------------------------

SCREENX	EQU	320
SCREENY	EQU	200

MAX_WALLS	EQU	192
MAX_SPRITES	EQU	64
MAX_OBJECTS	EQU	MAX_WALLS+MAX_SPRITES

M_PBC	EQU	$FFE0	; Port B Control Register
M_HSR	EQU	$FFE9	; Host Status Register
M_HRX	EQU	$FFEB	; Host Receive Data Register
M_HTX	EQU	$FFEB	; Host Transmit Data Register

; -------------	Macros pour envois/receptions

READHOST	MACRO	destination
	jclr	#0,X:<<M_HSR,*
	movep	X:<<M_HRX,destination
	ENDM

WRITEHOST	MACRO	source
	jclr	#1,X:<<M_HSR,*
	movep	source,X:<<M_HTX
	ENDM

PUSH	MACRO	source
	move	source,b
	move	b,x:(r0)+
	ENDM

POP	MACRO	destination
	move	x:(r0)+,a
	move	a,destination
	ENDM

; ---------------------------------------------
;	Programme principal
; ---------------------------------------------

	ORG	p:$0000	; origine $0000
	jmp	<start	; sauter les vecteurs
			; d'interruption
			
	ORG	p:$0040	; origine $0040

start	movep	#1,x:<<M_PBC	; Mode host interface
	move	#$ffff,m0
	move	m0,m1
	move	m0,m2
	move	m0,m3
	move	m0,m4
	move	m0,x:resx

; -------------	Boucle d'attente des commandes

attente	READHOST	a	; Lit un ordre

	move	#>"SCR",x0
	cmp	x0,a	#>"NOD",x0
	jeq	<BuildScene
	cmp	x0,a	#>"SEG",x0
	jeq	<EmpileNode
	cmp	x0,a	#>"SEC",x0
	jeq	<EmpileSeg
	cmp	x0,a	#>"LDF",x0
	jeq	<EmpileSector
	cmp	x0,a	#>"THI",x0
	jeq	<EmpileLinedef
	cmp	x0,a	#>"MUR",x0
	jeq	<EmpileThing

	cmp	x0,a	#>"MRT",x0
	jeq	<RenderMur
	cmp	x0,a	#>"MUP",x0
	jeq	<RenderMurT
	cmp	x0,a	#>"MDN",x0
	jeq	<RenderMurup
	cmp	x0,a	#>"SOL",x0
	jeq	<RenderMurdn
	cmp	x0,a	#>"PLA",x0
	jeq	<RenderSol
	cmp	x0,a	#>"SKU",x0
	jeq	<RenderPlafond
	cmp	x0,a	#>"SKD",x0
	jeq	<RenderSkyUp
	cmp	x0,a	#>"SPR",x0
	jeq	<RenderSkyDown
	cmp	x0,a	#>"DEJ",x0
	jeq	<RenderSprite

	cmp	x0,a	#>"INI",x0
	jeq	<EnvoieTraceDeja
	cmp	x0,a	#>"RDR",x0
	jeq	<InitRenderScene
	cmp	x0,a
	jeq	<RenderStackElement
	
	jmp	<attente

; -----------------------------------------------
;	Valeurs 48 bits
; -----------------------------------------------

	ORG	x:$0030
	ORG	y:$0030
	ORG	l:$0000

sauvegarde	ds	1
sauvegarde1	ds	1

; ---------------------------------------------
;	Routines mathmatiques
; ---------------------------------------------

	include	"div24.asm"
	include	"div24s.asm"
	include	"racine.asm"
	include	"arctan.asm"
	include	"lumiere.asm"

; -------------	Quelques variables

	ORG	x:
trace_deja	ds	1	; Nb de colonnes  ne pas tracer
draw_start	ds	1	; Adr du premier mur  tracer
	
; -----------------------------------------------
;	Envoie -1 si fin trac
; -----------------------------------------------

	ORG	p:
EnvoieTraceDeja
	move	x:trace_free1,a
	move	x:trace_free2,b
	cmp	a,b	x:trace_deja,b
	jpl	<ETD_saut
	WRITEHOST	#>-1
	jmp	<attente
ETD_saut
	clr	a	x:resx,y0
	cmp	y0,b	#>-1,y1
	teq	y1,a
	WRITEHOST	a
	jmp	<attente

; ----------------------------------------
;	Pour l'affichage des murs
; ----------------------------------------

	ORG	x:

to030_colstart	ds	1
to030_colend	ds	1
to030_x	ds	1
to030_y	ds	1
to030_ycoef	ds	1
to030_lum	ds	1

mur_xmapping	ds	1
mur_ymapping	ds	1

	ORG	l:

colonne_start	ds	1	; Y du haut du mur
colonne_scoef	ds	1	; Coef
colonne_end	ds	1	; Y du bas du mur
colonne_ecoef	ds	1	; Coef

; ----------------------------------------
;	Pour l'affichage du sol/plafond
; ----------------------------------------

	ORG	x:

ligne_encours	ds	1

ligne_dy	ds	1
ligne_depart	ds	1	; Y de depart (>=resy/2)
ligne_fin	ds	1	; Y de fin (<=resy-1)
ligne_hauteur	ds	1

	ORG	l:

ligne_start	ds	1	; X de depart
ligne_scoef	ds	1
ligne_end	ds	1	; X de fin
ligne_ecoef	ds	1

ligne_xs	ds	1	; Coefs de base pour le mapping
ligne_ys	ds	1
ligne_xc	ds	1
ligne_yc	ds	1

ligne_xs1	ds	1	; Coefs variables pour le mapping
ligne_ys1	ds	1
ligne_xc1	ds	1
ligne_yc1	ds	1

; -------------	Pour l'affichage des sprites

	ORG	x:
inverse_ycoef	ds	1

; -------------	Segment : mur ou sprite

	ORG	x:

seg_orig_x1	ds	1	; coordonnes source du SEG (16:8)
seg_orig_y1	ds	1
seg_orig_x2	ds	1
seg_orig_y2	ds	1

seg_x1	ds	1	; coordonnes dest, clippes (16:8)
seg_y1	ds	1
seg_x2	ds	1
seg_y2	ds	1

xe1	ds	1
xe2	ds	1
local_longueur	ds	1

; ------------------------------
;	Routines diverses
; ------------------------------

	include	"build\clip_seg.asm"
	include	"build\pile.asm"
	
	include	"build\build.asm"
	include	"build\node.asm"
	include	"build\seg.asm"
	include	"build\sector.asm"
	include	"build\linedef.asm"
	include	"build\thing.asm"

	include	"render\render.asm"
	include	"render\wall.asm"
	include	"render\sprite.asm"

	include	"render\mur.asm"
	include	"render\murup.asm"
	include	"render\murdn.asm"
	include	"render\murt.asm"
	include	"render\plafond.asm"
	include	"render\sol.asm"
	include	"render\skyup.asm"
	include	"render\skydn.asm"

; ----------------------------------------
;	Tableaux divers
; ----------------------------------------

	ORG	x:

trace_free1	ds	1	; Numero premiere colonne libre
trace_free2	ds	1	; Numero dernier colonne libre
trace_flag	ds	SCREENX	; 0 colonne libre, -1 colonne tracee
trace_ymin	ds	SCREENX
trace_ymax	ds	SCREENX
ciel_table	ds	SCREENX
trace_K	ds	SCREENX

; -------------	Tri des objets

sort_current	ds	1	; Nb objet dans pile OU *Nb objet en cours d'affichage

sort_tstart	ds	1
sort_table	ds	MAX_OBJECTS	; Table de tri (Nb d'objets)

sort_ystart	ds	1
sort_maxy	ds	MAX_OBJECTS	; Y max de chaque objet

; -------------	Pile de trace 

drawstack
DRAW_ELEMENT
DRAW_XE1	equ	*-DRAW_ELEMENT
	ds	1	; (24:0) premiere colonne
DRAW_XE2	equ	*-DRAW_ELEMENT
	ds	1	; (24:0) derniere colonne
DRAW_YE1	equ	*-DRAW_ELEMENT
	ds	1	; (24:0) premiere ligne
DRAW_YE2	equ	*-DRAW_ELEMENT
	ds	1	; (24:0) derniere ligne
DRAW_ORIG_X1	equ	*-DRAW_ELEMENT
	ds	1	; (16:8) coordonnes du mur avant clipping
DRAW_ORIG_Y1	equ	*-DRAW_ELEMENT
	ds	1	; (16:8)
DRAW_ORIG_X2	equ	*-DRAW_ELEMENT
	ds	1	; (16:8)
DRAW_ORIG_Y2	equ	*-DRAW_ELEMENT
	ds	1	; (16:8)
DRAW_X1	equ	*-DRAW_ELEMENT
	ds	1	; (16:8) coordonnes du mur apres clipping (1ere colonne,derniere colonne)
DRAW_Y1	equ	*-DRAW_ELEMENT
	ds	1	; (16:8)
DRAW_X2	equ	*-DRAW_ELEMENT
	ds	1	; (16:8)
DRAW_Y2	equ	*-DRAW_ELEMENT
	ds	1	; (16:8)
DRAW_CEILING1	equ	*-DRAW_ELEMENT
	ds	1	; (16:8) plafond 1
DRAW_FLOOR1	equ	*-DRAW_ELEMENT
	ds	1	; (16:8) sol 1
DRAW_CEILING2	equ	*-DRAW_ELEMENT
	ds	1	; (16:8) plafond 2
DRAW_FLOOR2	equ	*-DRAW_ELEMENT
	ds	1	; (16:8) sol 2
DRAW_LIGHT	equ	*-DRAW_ELEMENT
	ds	1	; (24:0) luminosite
DRAW_SINUS	equ	*-DRAW_ELEMENT
	ds	1	; (8:16) local_sinus
DRAW_COSINUS	equ	*-DRAW_ELEMENT
	ds	1	; (8:16) local_cosinus
DRAW_LDF_ATTRIB	equ	*-DRAW_ELEMENT
	ds	1	; (24:0) linedef_attrib
DRAW_SDF_OFFX	equ	*-DRAW_ELEMENT
	ds	1	; (24:0) offset x
DRAW_SDF_OFFY	equ	*-DRAW_ELEMENT
	ds	1	; (16:8) offset y
DRAW_PILE_TYPE	equ	*-DRAW_ELEMENT
	ds	1	; (24:0) type d'objet
SIZEOF_DRAW	equ	*-DRAW_ELEMENT
	ds	SIZEOF_DRAW*(MAX_OBJECTS-1)
drawstack_end
